collectedEntities = filteredSystem({ collected = T.PdImage })

local onCollidingRemove = { "mass", "velocity", "canCollideWith" }

roundSystem = filteredSystem({ roundAction = T.RoundStateAction, position = Maybe(T.XyPair) }, function(e, _, system)
    if e.roundAction == "end" then
        playdate.setAutoLockDisabled(false)

        local y = e.position.y - 240
        local rectWidth = 150
        local plateSize = { x = rectWidth, y = 10 }

        system.world:addEntity({
            drawAsRectangle = { size = plateSize },
            size = plateSize,
            mass = 0.5,
            velocity = { x = 0, y = 0 },
            position = { x = e.position.x - (rectWidth / 2), y = y },
            canCollideWith = 2,
            canBeCollidedBy = 2,
            isSolid = true,
            stopMovingOnCollision = true,
        })

        -- TODO: Big ol' numbers displaying how many ingredients were collected?
        -- TODO: Could layer ingredients in rows of three? Maybe just when it's higher?
        local delayPerDrop = 150
        local delay = 0
        for i, collectable in ipairs(collectedEntities.entities) do
            local collX, collY = collectable.collected:getSize()
            y = y - collY - 15
            playdate.timer.new(delay, function(ee, ccollX, ccollY, yy, ii, ssystem, ccollectable)
                ssystem.world:addEntity({
                    drawAsSprite = ccollectable.collected,
                    size = { x = ccollX, y = ccollY / 2 },
                    mass = 0.5,
                    velocity = { x = 0, y = 0 },
                    position = { x = ee.position.x - (ccollX / 2), y = yy },
                    canCollideWith = 2,
                    canBeCollidedBy = 2,
                    isSolid = true,
                    stopMovingOnCollision = true,
                    onCollidingRemove = onCollidingRemove,
                    focusOnCollide = ii,
                    expireBelowScreenBy = 5,
                })
            end, e, collX, collY, y, i, system, collectable)
            delay = delay + delayPerDrop
            system.world:removeEntity(collectable)
        end

        local availableUpgrades = Utils.getNDifferentValues(getAvailableSpawnerUpgrades(), 3)
        -- Sorting from shortest to longest sort of makes them look like a bun?
        table.sort(availableUpgrades, function(a, b)
            return #a.name > #b.name
        end)

        y = y - 50
        local menuEntity = {
            menuItems = {},
            canReceiveInput = T.marker,
        }
        local upgradeBelow
        for _, upgrade in ipairs(availableUpgrades) do
            local collX, collY = 75, 21
            y = y - collY - 15
            local upgradeEntity = {
                onSelect = upgrade.apply,
                drawAsText = {
                    text = upgrade.name,
                    style = TextStyle.Inverted,
                },
                size = { x = collX, y = collY },
                mass = 0.5,
                velocity = { x = 0, y = 0 },
                position = { x = e.position.x, y = y },
                canCollideWith = 2,
                canBeCollidedBy = 2,
                isSolid = true,
                stopMovingOnCollision = true,
                onCollidingRemove = onCollidingRemove,
                focusOnCollide = i,
                navigateDown = upgradeBelow,
                highlighted = true,
            }
            if upgradeBelow then
                upgradeBelow.navigateUp = upgradeEntity
                upgradeBelow.highlighted = false
                upgradeBelow.drawAsText.style = TextStyle.Bordered
            end
            upgradeBelow = upgradeEntity
            menuEntity.menuItems[#menuEntity.menuItems + 1] = upgradeEntity
            playdate.timer.new(delay, function(_system, _upgradeEntity)
                _system.world:addEntity(_upgradeEntity)
            end, system, upgradeEntity)
            delay = delay + delayPerDrop
            system.world:addEntity(menuEntity)
        end
        system.world:removeEntity(e)
    end
end)
